import {
  APIRequest,
  APIResponse,
  APIEncoder,
  APIDecoder,
  ParameterUtil
} from '@ucap-webmessenger/api';
import { DeviceType } from '@ucap-webmessenger/core';
import { MessageType } from '../types/message.type';
import { CategoryType } from '../types/category.type';

export interface DetailRequest extends APIRequest {
  userSeq: number;
  deviceType: DeviceType;
  tokenKey: string;
  type: MessageType;

  msgId: number;
}

export interface DetailResponse extends APIResponse {
  responseCode: string;
  responseMsg: string;

  msgInfo: {
    msgId: number;
    category: CategoryType;
    title: string;
    titleYn: boolean;
  };

  content: {
    type: MessageType;
    sendUserSeq: number;
    sendUserName: string;
    reservationTime: Date | null;
    sendYn: boolean;
    regDate: Date;
    attachmentYn: boolean;
    smsYn: boolean;
    fileAllow: string;
  }[];

  recvList: {
    userSeq: number;
    userName: string;
    cancelYn: boolean;
    readDate: Date;
    readYn: boolean;
  }[];
}

const DetailEncodeMap = {};

export const encodeDetail: APIEncoder<DetailRequest> = (req: DetailRequest) => {
  return ParameterUtil.encode(DetailEncodeMap, req);
};

export const decodeDetail: APIDecoder<DetailResponse> = (res: any) => {
  return {
    statusCode: res.StatusCode,
    errorMessage: res.ErrorMessage,

    responseCode: res.responseCode,
    responseMsg: res.responseMsg,

    msgInfo: null,

    content: [],

    recvList: []
  } as DetailResponse;
};

export interface ReadRequest extends APIRequest {
  userSeq: number;
  deviceType: DeviceType;
  tokenKey: string;
  type: MessageType;

  msgId: number;
}

export interface ReadResponse extends APIResponse {
  responseCode: string;
  responseMsg: string;
}

const ReadEncodeMap = {};

export const encodeRead: APIEncoder<ReadRequest> = (req: ReadRequest) => {
  return ParameterUtil.encode(ReadEncodeMap, req);
};

export const decodeRead: APIDecoder<ReadResponse> = (res: any) => {
  return {
    statusCode: res.StatusCode,
    errorMessage: res.ErrorMessage,

    responseCode: res.responseCode,
    responseMsg: res.responseMsg
  } as ReadResponse;
};

export interface ReadCheckResponse extends APIResponse {
  responseCode: string;
  responseMsg: string;

  recvList: {
    userSeq: number;
    userName: string;
    cancelYn: boolean;
    readDate: Date;
    readYn: boolean;
  }[];
}

export const decodeReadCheck: APIDecoder<ReadCheckResponse> = (res: any) => {
  return {
    statusCode: res.StatusCode,
    errorMessage: res.ErrorMessage,

    responseCode: res.responseCode,
    responseMsg: res.responseMsg,

    recvList: []
  } as ReadCheckResponse;
};

export interface CancelRequest extends APIRequest {
  userSeq: number;
  deviceType: DeviceType;
  tokenKey: string;
  type: MessageType;

  msgId: number;
  recvUserList: { userSeq: number }[];
}

export interface CancelResponse extends APIResponse {
  responseCode: string;
  responseMsg: string;
}

const CancelEncodeMap = {};

export const encodeCancel: APIEncoder<CancelRequest> = (req: CancelRequest) => {
  return ParameterUtil.encode(CancelEncodeMap, req);
};

export const decodeCancel: APIDecoder<CancelResponse> = (res: any) => {
  return {
    statusCode: res.StatusCode,
    errorMessage: res.ErrorMessage,

    responseCode: res.responseCode,
    responseMsg: res.responseMsg
  } as CancelResponse;
};

export interface CancelReservationRequest extends APIRequest {
  userSeq: number;
  deviceType: DeviceType;
  tokenKey: string;
  type: MessageType;

  msgId: number;
}

export interface CancelReservationResponse extends APIResponse {
  responseCode: string;
  responseMsg: string;
}

const CancelReservationEncodeMap = {};

export const encodeCancelReservation: APIEncoder<CancelReservationRequest> = (
  req: CancelReservationRequest
) => {
  return ParameterUtil.encode(CancelReservationEncodeMap, req);
};

export const decodeCancelReservation: APIDecoder<CancelReservationResponse> = (
  res: any
) => {
  return {
    statusCode: res.StatusCode,
    errorMessage: res.ErrorMessage,

    responseCode: res.responseCode,
    responseMsg: res.responseMsg
  } as CancelReservationResponse;
};

export interface RetrieveResourceFileRequest extends APIRequest {
  userSeq: number;
  deviceType: DeviceType;
  tokenKey: string;
  type: MessageType;

  msgId: number;
  resUrl: string;
}

export interface RetrieveResourceFileResponse extends APIResponse {}

const RetrieveResourceFileEncodeMap = {};

export const encodeRetrieveResourceFile: APIEncoder<RetrieveResourceFileRequest> = (
  req: RetrieveResourceFileRequest
) => {
  return ParameterUtil.encode(RetrieveResourceFileEncodeMap, req);
};

export const decodeRetrieveResourceFile: APIDecoder<RetrieveResourceFileResponse> = (
  res: any
) => {
  return {} as RetrieveResourceFileResponse;
};
