import {
  APIRequest,
  APIResponse,
  APIEncoder,
  APIDecoder,
  ParameterUtil
} from '@ucap-webmessenger/api';
import { DeviceType } from '@ucap-webmessenger/core';
import { MessageType } from '../types/message.type';
import { CategoryType } from '../types/category.type';
import { ContentType } from '../types/content.type';

export interface SendRequest extends APIRequest {
  userSeq: number;
  deviceType: DeviceType;
  tokenKey: string;
  type: MessageType;

  userName: string;
  category: CategoryType;
  title: string;
  titleYn: boolean;
  listOrder: ContentType[];
  reservationTime: string;
  smsYn: boolean;

  textContent: { text: string }[];
  recvUserList: { userSeq: number; userName: string }[];
}

export interface SendResponse extends APIResponse {
  responseCode: string;
  responseMsg: string;
}

const SendEncodeMap = {};

export const encodeSend: APIEncoder<SendRequest> = (req: SendRequest) => {
  return ParameterUtil.encode(SendEncodeMap, req);
};

export const decodeSend: APIDecoder<SendResponse> = (res: any) => {
  return {
    statusCode: res.StatusCode,
    errorMessage: res.ErrorMessage,

    responseCode: res.responseCode,
    responseMsg: res.responseMsg
  } as SendResponse;
};
